@@ -20,6 +20,7 @@ use std::ffi::{c_void, CStr, CString};
 use std::fmt::{self, Display, Formatter};
 use std::marker::{self, PhantomData};
 use std::mem;
+use std::os::fd::{IntoRawFd, OwnedFd};
 use std::os::unix::prelude::{AsRawFd, RawFd};
 use std::ptr;
 use std::result;
@@ -130,11 +131,6 @@ pub(crate) trait ResolveWireEvent: Sized {
     /// Resolve a pointer to `xcb_generic_event_t` to `Self`, inferring the correct subtype
     /// using `response_type` field and `first_event`
     ///
-    /// # Panics
-    /// Panics if the event subtype cannot be resolved to `Self`. That is,
-    /// `response_type` field must be checked beforehand to be in range with
-    /// `first_event`.
-    ///
     /// # Safety
     /// `event` must be a valid, non-null event returned by `xcb_wait_for_event`
     /// or similar function
@@ -163,19 +159,14 @@ pub(crate) trait ResolveWireGeEvent: Sized {
 ///
 /// `Self` is normally an enum of several event subtypes.
 /// See [crate::x::Error] and [crate::ProtocolError]
-pub(crate) trait ResolveWireError {
+pub(crate) trait ResolveWireError: Sized {
     /// Convert a pointer to `xcb_generic_error_t` to `Self`, inferring the correct subtype
     /// using `response_type` field and `first_error`.
     ///
-    /// # Panics
-    /// Panics if the error subtype cannot be resolved for `self`. That is,
-    /// `response_type` field must be checked beforehand to be in range with
-    /// `first_error`.
-    ///
     /// # Safety
     /// `err` must be a valid, non-null error obtained by `xcb_wait_for_reply`
     /// or similar function
-    unsafe fn resolve_wire_error(first_error: u8, error: *mut xcb_generic_error_t) -> Self;
+    unsafe fn resolve_wire_error(first_error: u8, error: *mut xcb_generic_error_t) -> Option<Self>;
 }
 
 /// Trait for types that can serialize themselves to the X wire.
@@ -311,6 +302,13 @@ pub trait Reply {
     /// The returned pointer must be freed with [`libc::free`] to avoid any memory leak, or be used
     /// to build another reply.
     unsafe fn into_raw(self) -> *const u8;
+
+    /// Get the raw pointer of the XCB reply.
+    ///
+    /// # Safety
+    /// The returned pointer must NOT be freed. Passing this pointer to [`libc::free`] will result in a double free
+    /// when the reply is dropped.
+    unsafe fn as_raw(&self) -> *const u8;
 }
 
 /// General trait for cookies returned by requests.
@@ -829,7 +827,10 @@ impl Connection {
     /// Connects to an X server, given the open socket fd and the
     /// `auth_info`. The file descriptor `fd` is bidirectionally connected to an X server.
     /// If the connection should be unauthenticated, `auth_info` must be `None`.
+    #[deprecated(note = "unsound API, use `connect_with_fd` instead")]
+    #[doc(hidden)]
     pub fn connect_to_fd(fd: RawFd, auth_info: Option<AuthInfo<'_>>) -> ConnResult<Self> {
+        #[allow(deprecated)]
         Self::connect_to_fd_with_extensions(fd, auth_info, &[], &[])
     }
 
@@ -844,6 +845,8 @@ impl Connection {
     ///
     /// # Panics
     /// Panics if one of the mandatory extension is not present.
+    #[deprecated(note = "unsound API, use `connect_with_fd_and_extensions` instead")]
+    #[doc(hidden)]
     pub fn connect_to_fd_with_extensions(
         fd: RawFd,
         auth_info: Option<AuthInfo<'_>>,
@@ -880,6 +883,64 @@ impl Connection {
         conn.has_error().map(|_| conn)
     }
 
+    /// Connects to the X server with an open socket file descriptor and optional authentification info.
+    ///
+    /// Connects to an X server, given the open socket fd and the
+    /// `auth_info`. The file descriptor `fd` is bidirectionally connected to an X server.
+    /// If the connection should be unauthenticated, `auth_info` must be `None`.
+    pub fn connect_with_fd(fd: OwnedFd, auth_info: Option<AuthInfo<'_>>) -> ConnResult<Self> {
+        Self::connect_with_fd_and_extensions(fd, auth_info, &[], &[])
+    }
+
+    /// Connects to the X server with an open socket file descriptor and optional authentification info.
+    ///
+    /// Extension data specified by `mandatory` and `optional` is cached to allow
+    /// the resolution of events and errors in these extensions.
+    ///
+    /// Connects to an X server, given the open socket fd and the
+    /// `auth_info`. The file descriptor `fd` is bidirectionally connected to an X server.
+    /// If the connection should be unauthenticated, `auth_info` must be `None`.
+    ///
+    /// # Panics
+    /// Panics if one of the mandatory extension is not present.
+    pub fn connect_with_fd_and_extensions(
+        fd: OwnedFd,
+        auth_info: Option<AuthInfo<'_>>,
+        mandatory: &[Extension],
+        optional: &[Extension],
+    ) -> ConnResult<Self> {
+        let mut auth_info = auth_info.map(|auth_info| {
+            let auth_name = CString::new(auth_info.name).unwrap();
+            let auth_data = CString::new(auth_info.data).unwrap();
+
+            let auth_info = xcb_auth_info_t {
+                namelen: auth_name.as_bytes().len() as _,
+                name: auth_name.as_ptr() as *mut _,
+                datalen: auth_data.as_bytes().len() as _,
+                data: auth_data.as_ptr() as *mut _,
+            };
+            // return the strings too otherwise they would drop
+            (auth_info, auth_name, auth_data)
+        });
+
+        let ai_ptr = if let Some(auth_info) = auth_info.as_mut() {
+            &mut auth_info.0 as *mut _
+        } else {
+            ptr::null_mut()
+        };
+
+        let fd = fd.into_raw_fd();
+
+        let conn = unsafe {
+            let conn = xcb_connect_to_fd(fd, ai_ptr);
+            check_connection_error(conn)?;
+
+            Self::from_raw_conn_and_extensions(conn, mandatory, optional)
+        };
+
+        conn.has_error().map(|_| conn)
+    }
+
     /// Connects to the X server, using an authorization information.
     ///
     /// Connects to the X server specified by `display_name`, using the
@@ -1095,6 +1156,14 @@ impl Connection {
         self.ext_data.iter().map(|eed| eed.ext)
     }
 
+    /// Get the data of the extensions activated for this connection.
+    ///
+    /// You may use this to manually resolve an event or an error with
+    /// `xcb::event::resolve_event` or `xcb::error::resolve_error`.
+    pub fn active_extensions_data(&self) -> &[ExtensionData] {
+        &self.ext_data
+    }
+
     /// Returns the inner ffi `xcb_connection_t` pointer
     pub fn get_raw_conn(&self) -> *mut xcb_connection_t {
         self.c
@@ -1218,6 +1287,16 @@ impl Connection {
         event::resolve_event(ev, &self.ext_data)
     }
 
+    /// Resolve an xcb_generic_error_t pointer into an Error.
+    /// # Safety
+    /// The caller is repsonsible to ensure that the `err` pointer is not NULL.
+    /// The ownership of the pointer is effectively transferred to the
+    /// returned Error and it will be destroyed when the Error is
+    /// dropped.
+    pub unsafe fn resolve_error(&self, err: &mut xcb_generic_error_t) -> error::ProtocolError {
+        error::resolve_error(err, &self.ext_data)
+    }
+
     /// Blocks and returns the next event or error from the server.
     ///
     /// # Example
@@ -1309,9 +1388,9 @@ impl Connection {
         unsafe {
             let ext: *mut xcb_extension_t = match XGE::EXTENSION {
                 #[cfg(feature = "xinput")]
-                Extension::Input => &mut xinput::FFI_EXT as *mut _,
+                Extension::Input => ptr::addr_of_mut!(xinput::FFI_EXT),
                 #[cfg(feature = "present")]
-                Extension::Present => &mut present::FFI_EXT as *mut _,
+                Extension::Present => ptr::addr_of_mut!(present::FFI_EXT),
                 _ => unreachable!("only Input and Present have XGE events"),
             };
 
@@ -1370,10 +1449,10 @@ impl Connection {
         unsafe {
             let ext: *mut xcb_extension_t = match extension {
                 #[cfg(feature = "xinput")]
-                Extension::Input => &mut xinput::FFI_EXT as *mut _,
+                Extension::Input => ptr::addr_of_mut!(xinput::FFI_EXT),
                 #[cfg(feature = "present")]
-                Extension::Present => &mut present::FFI_EXT as *mut _,
-                _ => panic!("only Input and Present have XGE events"),
+                Extension::Present => ptr::addr_of_mut!(present::FFI_EXT),
+                _ => unreachable!("only Input and Present have XGE events"),
             };
 
             let raw = xcb_register_for_special_xge(self.c, ext, eid.resource_id(), ptr::null_mut());
@@ -2063,17 +2142,6 @@ pub(crate) fn align_pad(base: usize, align: usize) -> usize {
     (-base & (align - 1)) as usize
 }
 
-/// Get a value from a pointer, using ptr::copy_nonoverlapping to avoid alignment issues
-///
-/// # Safety
-/// The pointer must point to a valid `T` value
-#[inline]
-pub(crate) unsafe fn value_from_ptr<T>(ptr: *const T) -> T {
-    let mut val = mem::MaybeUninit::<T>::uninit();
-    ptr::copy_nonoverlapping(ptr, val.as_mut_ptr(), 1);
-    val.assume_init()
-}
-
 #[test]
 fn test_align_pad() {
     // align 1
